<!--
    DO NOT MANUALLY EDIT THIS FILE
    THIS FILE IS AUTOMATICALLY GENERATED WITH resilient-circuits codegen
-->

# {{ name }}

{% for fn in wf_functions %}
## Function - {{ fn.display_name }}

### API Name
`{{ fn.name }}`

### Output Name
`{{ fn.result_name }}`

### Message Destination
`{{ fn.message_destination }}`

### Pre-Processing Script
```python
{{ fn.pre_processing_script }}
```

### Post-Processing Script
```python
{{ fn.post_processing_script }}
```

---

{% endfor %}