# Tox (https://tox.readthedocs.io/) is a tool for running tests
# in multiple virtualenvs. This configuration file will run the
# test suite on all supported python versions. To use it, "pip install tox"
# and then run "tox" from this directory.

[tox]
envlist = py27, py36
skip_missing_interpreters=True

[testenv]
passenv=TEST_RESILIENT_*
commands = pytest -s {posargs}
deps =
    pytest
    mock
    resilient
    resilient-circuits
    pytest-resilient-circuits
